#include "stdafx.h"
#include "artfile.h"
#include <string.h>

ArtFile::ArtFile(FILESTRUCT* data)
{
	if(data == NULL)
	{
		m_nStatus = BL_ERROR;
		return;
	}

	//Make sure our data is initialized	
	m_Art.Clear();

	m_nStatus = BL_OK;
	
	uint32 nMemOffset = 0; 
	memcpy(&m_Art.nVersion, &data->data[nMemOffset], 4);
	nMemOffset += (sizeof(char) * 4);

	//This value gets discarded later for the real value.
	memcpy(&m_Art.nNumTiles, &data->data[nMemOffset], 4);
	nMemOffset += (sizeof(char) * 4);

	memcpy(&m_Art.nTileStart , &data->data[nMemOffset], 4);
	nMemOffset += (sizeof(char) * 4);

	memcpy(&m_Art.nTileEnd , &data->data[nMemOffset], 4);
	nMemOffset += (sizeof(char) * 4);

	//Calculate the real value of the tile count
	m_Art.nNumTiles = (m_Art.nTileEnd - m_Art.nTileStart) + 1;

	/*
	memcpy(&m_Art.tilesizex[0] , &data->data[nMemOffset], (m_Art.nNumTiles * 2));
	nMemOffset += (sizeof(char) * (m_Art.nNumTiles * 2));
	*/
	
	m_Art.tilesizex = new uint16[m_Art.nNumTiles];
	memcpy(&m_Art.tilesizex[0] , &data->data[nMemOffset], (m_Art.nNumTiles * 2));
	nMemOffset += (sizeof(char) * (m_Art.nNumTiles * 2));

	m_Art.tilesizey = new uint16[m_Art.nNumTiles];
	memcpy(&m_Art.tilesizey[0] , &data->data[nMemOffset], (m_Art.nNumTiles * 2));
	nMemOffset += (sizeof(char) * (m_Art.nNumTiles * 2));

	m_Art.picanm = new uint32[m_Art.nNumTiles];
	memcpy(&m_Art.picanm[0] , &data->data[nMemOffset], (m_Art.nNumTiles * 4));
	nMemOffset += (sizeof(char) * (m_Art.nNumTiles * 4));	
	
	//Create or tile array
	m_Art.tiles = new TILESTRUCT[m_Art.nNumTiles];

	uint32 i = 0;
	for(i = 0; i < m_Art.nNumTiles; i++)
	{
		m_Art.tiles[i].tilesizex = m_Art.tilesizex[i];
		m_Art.tiles[i].tilesizey = m_Art.tilesizey[i];
		m_Art.tiles[i].picanm = m_Art.picanm[i];		
	}

	//delete our temp data
	if(m_Art.tilesizex)
	{
		delete[] m_Art.tilesizex;
	}

	if(m_Art.tilesizey)
	{
		delete[] m_Art.tilesizey;
	}

	if(m_Art.picanm)
	{
		delete[] m_Art.picanm;
	}

	//Read our actual tile data now
	for(i = 0; i < m_Art.nNumTiles; i++)
	{
		uint32 nSize = m_Art.tiles[i].tilesizex * m_Art.tiles[i].tilesizey;
		if( nSize > 0 )
		{
			m_Art.tiles[i].data = new uint8[nSize];
			memcpy(&m_Art.tiles[i].data[0] , &data->data[nMemOffset], nSize);
			nMemOffset += (sizeof(char) * nSize);

		}
		else
		{
			m_Art.tiles[i].data = NULL;
		}
	}


}


ArtFile::ArtFile(const char* sFilename)
{
	//Make sure our data is initialized
	m_Art.Clear();

	m_nStatus = BL_OK;

	FILE *fp = fopen(sFilename, "rb");
	if(fp != NULL)
	{
		//Read contents of the Art file.
		fread(&m_Art.nVersion, 4, 1, fp);
		//This value gets discarded later for the real value.
		fread(&m_Art.nNumTiles, 4, 1, fp);

		fread(&m_Art.nTileStart, 4, 1, fp);
		fread(&m_Art.nTileEnd, 4, 1, fp);

		m_Art.nNumTiles = (m_Art.nTileEnd - m_Art.nTileStart) + 1;

		m_Art.tilesizex = new uint16[m_Art.nNumTiles];
		fread(&m_Art.tilesizex[0], 2, m_Art.nNumTiles, fp);

		m_Art.tilesizey = new uint16[m_Art.nNumTiles];
		fread(&m_Art.tilesizey[0], 2, m_Art.nNumTiles, fp);

		m_Art.picanm = new uint32[m_Art.nNumTiles];
		fread(&m_Art.picanm[0], 4, m_Art.nNumTiles, fp);

		//Create or tile array
		m_Art.tiles = new TILESTRUCT[m_Art.nNumTiles];

		uint32 i = 0;
		for(i = 0; i < m_Art.nNumTiles; i++)
		{
			m_Art.tiles[i].tilesizex = m_Art.tilesizex[i];
			m_Art.tiles[i].tilesizey = m_Art.tilesizey[i];
			m_Art.tiles[i].picanm = m_Art.picanm[i];		
		}

		//delete our temp data
		if(m_Art.tilesizex)
		{
			delete[] m_Art.tilesizex;
		}

		if(m_Art.tilesizey)
		{
			delete[] m_Art.tilesizey;
		}

		if(m_Art.picanm)
		{
			delete[] m_Art.picanm;
		}

		//Read our actual tile data now
		for(i = 0; i < m_Art.nNumTiles; i++)
		{
			uint32 nSize = m_Art.tiles[i].tilesizex * m_Art.tiles[i].tilesizey;
			if( nSize > 0 )
			{
				m_Art.tiles[i].data = new uint8[nSize];
				fread(&m_Art.tiles[i].data[0], nSize, 1, fp);
			}
			else
			{
				m_Art.tiles[i].data = NULL;
			}
		}

		fclose(fp);
	}
	else
	{
		m_nStatus = BL_ERROR;
	}
}

ArtFile::ArtFile()
{
	//Make sure our data is initialized
	m_Art.Clear();

	m_nStatus = BL_OK;
}


ArtFile::~ArtFile()
{

	if(m_Art.tiles)
	{
		for(uint32 i = 0; i <m_Art.nNumTiles; i++)
		{
			//delete each tile's data
			if( m_Art.tiles[i].data )
			{
				delete[] m_Art.tiles[i].data;
			}
		}
		delete[] m_Art.tiles;
	}

	/*
	if(m_Art.tilesizex)
	{
		delete[] m_Art.tilesizex;
	}

	if(m_Art.tilesizey)
	{
		delete[] m_Art.tilesizey;
	}

	if(m_Art.picanm)
	{
		delete[] m_Art.picanm;
	}
	*/

}

uint16 ArtFile::GetTileSizeX(uint32 nNum)
{
	if( nNum < ((GetTileEnd() - GetTileStart()) + 1) )
	{
		return m_Art.tiles[nNum].tilesizex; //m_Art.tilesizex[nNum];
	}

	return 0;
}

uint16 ArtFile::GetTileSizeY(uint32 nNum)
{
	if( nNum < ((GetTileEnd() - GetTileStart()) + 1) )
	{
		return m_Art.tiles[nNum].tilesizey;		
	}

	return 0;
}
