// buildlib.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include <stdio.h>
#include <memory.h>
#include <string.h>
#include "fileformatdefs.h"
#include "groupfile.h"
#include "mapfile.h"
#include "artfile.h"
#include "palettefile.h"

//
//
// Usage: buildlib -grp duke3d.grp
//
// Options:
// -g <filename> Specify Group File
// -e <filename> Extract
// -a <filename> Add File
// 
int ExtractAll(GroupFile *pGRP);
int TestGroupAccess(char* argv[]);
int TestGroupSave(GroupFile *pGRP, const char* sFilename);
int TestDiskAccess();

int main(int argc, char* argv[])
{

//
//Test disk access
//
return TestDiskAccess();
//

//
//Test Group Access
//
//return TestGroupAccess(argv);

//
// Test Group Save
//
/*
GroupFile pGrpFile = GroupFile(argv[1]);
if( pGrpFile.GetStatus() == 0)
{
	return TestGroupSave(&pGrpFile, "TEST.GRP");
}
*/


//
//Extract All
//GroupFile* pGrpFile = new GroupFile(argv[1]);
//if( pGrpFile->GetStatus() == 0)
//{
	//ExtractAll(pGrpFile);
//}


	return 0;
}


int ExtractAll(GroupFile *pGRP)
{

	uint32 nNumFiles = pGRP->GetNumFiles();

	for(uint32 i = 0; i < nNumFiles; i++)
	{				
		FILESTRUCT *pFileEntry = pGRP->GetFileEntry(i);

		if(pFileEntry)
		{

			//extract the file
			char sFilename[128];
			sprintf(sFilename, "data\\");
			strncat(sFilename, pFileEntry->fileheader.name, 12);

			FILE* fp;
			fp = fopen(sFilename, "wb");
			if(fp != NULL)
			{
				fwrite(&pFileEntry->data[0], sizeof(char), pFileEntry->fileheader.size, fp);
				fflush(fp);
				fclose(fp);
			}

		}
	}

	return 0;
}


int TestGroupAccess(char* argv[])
{

	GroupFile* pGrpFile = new GroupFile(argv[1]);
	if( pGrpFile->GetStatus() == 0)
	{
		printf("GRP Filename: %s\n", pGrpFile->GetFileName());
		printf("Number of files in this archive: %d\n", pGrpFile->GetNumFiles());
		uint32 nNumFiles = pGrpFile->GetNumFiles();

		for(uint32 i = 0; i < nNumFiles; i++)
		{

			FILESTRUCT *pFileEntry = pGrpFile->GetFileEntry(i);

			if(pFileEntry)
			{
				char sPrintableFilename[13];
				memset(sPrintableFilename, 0, 12);
				strncpy(sPrintableFilename, pFileEntry->fileheader.name, 12);
				sPrintableFilename[12] = '\0';

				//Print out some info on each file entry
				//printf("[%d] %s  Size: %d\n", i, sPrintableFilename, pFileEntry->fileheader.size);


				//
				// Test the .ART support
				//
				if( strcmp("TILES000.ART", sPrintableFilename) == 0)
				{
					ArtFile _artfile(pFileEntry);

					printf("\n------------------\n");
					printf("  ART FILE TEST\n");
					printf("------------------\n");
					printf("ArtFile: %s  Size: %d\n", sPrintableFilename, pFileEntry->fileheader.size);
					printf("Version: %d \n", _artfile.GetVersion());
					printf("NumTiles: %d\n", _artfile.GetNumTiles());
					printf("TileStart: %d\n", _artfile.GetTileStart());
					printf("TileEnd: %d\n", _artfile.GetTileEnd());

					printf("\n");

					/*
					//print each tiles dims
					for(uint32 j = 0; j < _artfile->GetNumTiles(); j++)
					{
					printf("Tile [%d] X: %d  Y: %d\n", j, _artfile->GetTileSizeX(j), _artfile->GetTileSizeY(j));
					}
					*/


					//if(_artfile)
					//delete _artfile;
				}

				//
				// Test the .MAP support
				//
				if( strcmp("E1L8.MAP", pFileEntry->fileheader.name) == 0)
				{

					MapFile *_map = new MapFile(pFileEntry);

					printf("------------------\n");
					printf("  MAP FILE TEST\n");
					printf("------------------\n");
					printf("Map: %s  Size: %d\n", sPrintableFilename, pFileEntry->fileheader.size);
					printf("Version: %d\n", _map->GetVersion());
					Pos pos = _map->GetStartPosition();
					printf("Start position: %d %d %d\n", pos.x, pos.y, pos.z);
					printf("Start Angle: %d\n", _map->GetStartAngle());
					printf("Startpoint Sector: %d\n", _map->GetStartSectorNum());						
					printf("Sectors: %d\n", _map->GetNumSectors());
					printf("Walls: %d\n", _map->GetNumWalls());
					printf("Sprites: %d\n", _map->GetNumSprites());

					delete _map;

				}


				if( strcmp("PALETTE.DAT", pFileEntry->fileheader.name) == 0)
				{

					PaletteFile _palette(pFileEntry);

					printf("---------------------\n");
					printf("  PALETTE FILE TEST\n");
					printf("---------------------\n");	
					printf("Palette: %s  Size: %d\n", sPrintableFilename, pFileEntry->fileheader.size);						
					printf("Number of palette lookups: : %d\n", _palette.GetNumPalLookups());

					//if(_palette)
					//	delete _palette;
				}

			}

		}
		/*
		// Save Grp file
		pGrpFile->SetFilename("test.grp");
		pGrpFile->Save();
		*/
	}

	if(pGrpFile)
	{
		delete pGrpFile;
	}

	return 0;
}

int TestGroupSave(GroupFile *pGRP, const char* sFilename)
{
	pGRP->SetFilename(sFilename);
	return pGRP->Save();
}

int TestDiskAccess()
{

	//Test loading art file from disk:
	ArtFile _artfile("data\\TILES000.ART");
	if(_artfile.GetStatus() == BL_ERROR)
	{
		printf("Error opening Art file!!\n");
		_artfile.~ArtFile();
		return 1;
	}


	printf("\n------------------\n");
	printf("  ART FILE TEST\n");
	printf("------------------\n");
	//printf("ArtFile: %s  Size: %d\n", "data\\TILES000.ART", pFileEntry->fileheader.size);
	printf("Version: %d \n", _artfile.GetVersion());
	printf("NumTiles: %d\n", _artfile.GetNumTiles());
	printf("TileStart: %d\n", _artfile.GetTileStart());
	printf("TileEnd: %d\n", _artfile.GetTileEnd());

	printf("\n\n");
	

	
	MapFile _map("data\\E1L8.MAP");
	if(_map.GetStatus() == BL_ERROR)
	{
		printf("Error opening Map file!!\n");
		//_artfile.~ArtFile();
		return 1;
	}

	printf("------------------\n");
	printf("  MAP FILE TEST\n");
	printf("------------------\n");
	//printf("Map: %s  Size: %d\n", sPrintableFilename, pFileEntry->fileheader.size);
	printf("Version: %d\n", _map.GetVersion());
	Pos pos = _map.GetStartPosition();
	printf("Start position: %d %d %d\n", pos.x, pos.y, pos.z);
	printf("Start Angle: %d\n", _map.GetStartAngle());
	printf("Startpoint Sector: %d\n", _map.GetStartSectorNum());						
	printf("Sectors: %d\n", _map.GetNumSectors());
	printf("Walls: %d\n", _map.GetNumWalls());
	printf("Sprites: %d\n\n\n", _map.GetNumSprites());

	//delete _map;

	PaletteFile _palette("data\\PALETTE.DAT");
	if(_palette.GetStatus() == BL_ERROR)
	{
		printf("Error opening Palette file!!\n");
		//_artfile.~ArtFile();
		return 1;
	}

	printf("---------------------\n");
	printf("  PALETTE FILE TEST\n");
	printf("---------------------\n");	
	//printf("Palette: %s  Size: %d\n", sPrintableFilename, pFileEntry->fileheader.size);						
	printf("Number of palette lookups: %d\n", _palette.GetNumPalLookups());

	PALETTESTRUCT* pPal = _palette.GetDataHandle();
	pPal->numpalookups = 123;
	printf("Number of palette lookups: %d\n", _palette.GetNumPalLookups());

	return 0;
}
