#ifndef _MapFile_H_
#define _MapFile_H_

#include "stdafx.h"
#include "fileformatdefs.h"
#include "vardefs.h"
#include <stdio.h>
#include "basefile.h"

/*! This is used to deal with \b MAP files */
class MapFile : public BaseFile
{
public:

	/*! Read a \b MAP file straight from the GroupFile */
	MapFile(FILESTRUCT* data);

	/*! Load a \b MAP file from disk */
	MapFile(const char* sFilename);

	/*! Create an empty \b MAP file */
	MapFile();

	~MapFile();

	/*! Returns the \b MAP format version */
	uint32 GetVersion(){ return m_Map.nVersion; }

	/*! Returns the player start position */
	Pos GetStartPosition(){ return m_Map.StartPosition; }

	/*! Returns the player start angle */
	uint16 GetStartAngle(){ return m_Map.nAngle; }

	/*! Returns the sector number of the player start position */
	uint16 GetStartSectorNum(){ return m_Map.nStartPointSecNum; }
	
	/*! Returns the total number of sectors in the map */
	uint16 GetNumSectors(){ return m_Map.nNumSectors; }

	/*! Returns the total number of walls in the map */
	uint16 GetNumWalls(){ return m_Map.nNumWalls; }

	/*! Returns the total number of sprites in the level */
	uint16 GetNumSprites(){ return m_Map.nNumSprites; }

	/*! Returns the formated \em MAPSTRUCT data */
	MAPSTRUCT* GetDataHandle(){ return &m_Map; }

protected:

	/*! The formated \b MAP data */
	MAPSTRUCT m_Map;

};

#endif
